@@ -350,7 +350,7 @@ static pj_bool_t ssl_on_data_read(pj_ssl_sock_t *ssock,
                 }
 
                 pj_sockaddr_print((pj_sockaddr_t*)&info.local_addr, buf, sizeof(buf), 1);
-                PJ_LOG(3, ("", "...%s successfully recv %d bytes echo", buf, st->recv));
+                PJ_LOG(3, ("", "...%s successfully recv %lu bytes echo", buf, st->recv));
                 st->done = PJ_TRUE;
             }
         }
@@ -501,7 +501,7 @@ static int https_client_test(unsigned ms_timeout)
     }
 
     PJ_LOG(3, ("", "...Done!"));
-    PJ_LOG(3, ("", ".....Sent/recv: %d/%d bytes", state.sent, state.recv));
+    PJ_LOG(3, ("", ".....Sent/recv: %lu/%lu bytes", state.sent, state.recv));
 
 on_return:
     if (ssock && !state.err && !state.done) 
@@ -755,7 +755,7 @@ static int echo_test(pj_ssl_sock_proto srv_proto, pj_ssl_sock_proto cli_proto,
     }
 
     PJ_LOG(3, ("", "...Done!"));
-    PJ_LOG(3, ("", ".....Sent/recv: %d/%d bytes", state_cli.sent, state_cli.recv));
+    PJ_LOG(3, ("", ".....Sent/recv: %lu/%lu bytes", state_cli.sent, state_cli.recv));
 
 on_return:
 #if (PJ_SSL_SOCK_IMP == PJ_SSL_SOCK_IMP_DARWIN) || \
@@ -1207,8 +1207,11 @@ static int server_non_ssl(unsigned ms_timeout)
     PJ_LOG(3, ("", "...Done!"));
 
 on_return:
-    if (asock_serv)
+    if (asock_serv) {
         pj_activesock_close(asock_serv);
+    } else if (sock != PJ_INVALID_SOCKET) {
+        pj_sock_close(sock);
+    }
     if (ssock_cli && !state_cli.err && !state_cli.done)
         pj_ssl_sock_close(ssock_cli);
     if (timer)
@@ -1448,7 +1451,7 @@ static int perf_test(unsigned clients, unsigned ms_handshake_timeout)
         pj_gettimeofday(&stop);
         PJ_TIME_VAL_SUB(stop, start);
 
-        PJ_LOG(3, ("", ".....Setup & data transfer duration: %d.%03ds", stop.sec, stop.msec));
+        PJ_LOG(3, ("", ".....Setup & data transfer duration: %ld.%03lds", stop.sec, stop.msec));
     }
 
     /* Check clients status */
@@ -1461,7 +1464,7 @@ static int perf_test(unsigned clients, unsigned ms_handshake_timeout)
     }
 
     PJ_LOG(3, ("", ".....Clients: %d (%d errors)", clients, cli_err));
-    PJ_LOG(3, ("", ".....Total sent/recv: %d/%d bytes", tot_sent, tot_recv));
+    PJ_LOG(3, ("", ".....Total sent/recv: %lu/%lu bytes", tot_sent, tot_recv));
 
 on_return:
     if (ssock_serv) 
